@@ -169,6 +169,62 @@ def _parse_url(url):
     return (hostname, port, resource, is_secure)
 
 
+DEFAULT_NO_PROXY_HOST = ["localhost", "127.0.0.1"]
+
+def _is_no_proxy_host(hostname, no_proxy):
+    if not no_proxy:
+        v = os.environ.get("no_proxy", "").replace(" ", "")
+        no_proxy = v.split(",")
+    if not no_proxy:
+        no_proxy = DEFAULT_NO_PROXY_HOST
+
+    return hostname in no_proxy
+
+def _get_proxy_info(hostname, is_secure, **options):
+    """
+    try to retrieve proxy host and port from environment if not provided in options.
+    result is (proxy_host, proxy_port, proxy_auth).
+    proxy_auth is tuple of username and password of proxy authentication information.
+    
+    hostname: websocket server name.
+
+    is_secure:  is the connection secure? (wss)
+                looks for "https_proxy" in env before falling back to "http_proxy"
+
+    options:    "http_proxy_host" - http proxy host name.
+                "http_proxy_port" - http proxy port.
+                "http_no_proxy"   - host names, which doesn't use proxy.
+                "http_proxy_auth" - http proxy auth infomation. tuple of username and password.
+                                    defualt is None
+    """
+    if _is_no_proxy_host(hostname, options.get("http_no_proxy", None)):
+        return None, 0, None
+
+    http_proxy_host = options.get("http_proxy_host", None)
+    if http_proxy_host:
+        return http_proxy_host, options.get("http_proxy_port", 0), options.get("http_proxy_auth", None)
+
+    env_keys = ["http_proxy"]
+    if is_secure:
+        env_keys.insert(0, "https_proxy")
+
+    for key in env_keys:
+        value = os.environ.get(key, None)
+        if value:
+            proxy = urlparse(value)
+            auth = (proxy.username, proxy.password) if proxy.username else None
+            return proxy.hostname, proxy.port, auth
+
+    return None, 0, None
+
+def _extract_err_message(exception):
+        message = getattr(exception, 'strerror', '')
+        if not message:
+            message = getattr(exception, 'message', '')
+
+        return message
+
+
 def create_connection(url, timeout=None, **options):
     """
     connect to url and return websocket object.
@@ -192,6 +248,9 @@ def create_connection(url, timeout=None, **options):
              "cookie" -> cookie value.
              "http_proxy_host" - http proxy host name.
              "http_proxy_port" - http proxy port. If not set, set to 80.
+             "http_no_proxy"   - host names, which doesn't use proxy.
+             "http_proxy_auth" - http proxy auth infomation. tuple of username and password.
+                                    defualt is None
              "enable_multithread" -> enable lock for multithread.
              "sockopt" -> socket options
              "sslopt" -> ssl option
@@ -400,12 +459,15 @@ class WebSocket(object):
                  "cookie" -> cookie value.
                  "http_proxy_host" - http proxy host name.
                  "http_proxy_port" - http proxy port. If not set, set to 80.
+                 "http_no_proxy"   - host names, which doesn't use proxy.
+                 "http_proxy_auth" - http proxy auth infomation. tuple of username and password.
+                                    defualt is None
                  "subprotocols" - array of available sub protocols. default is None.
 
         """
 
         hostname, port, resource, is_secure = _parse_url(url)
-        proxy_host, proxy_port = options.get("http_proxy_host", None), options.get("http_proxy_port", 0)
+        proxy_host, proxy_port, proxy_auth = _get_proxy_info(hostname, is_secure, **options)
         if not proxy_host:
             addrinfo_list = socket.getaddrinfo(hostname, port, 0, 0, socket.SOL_TCP)
         else:
@@ -441,7 +503,7 @@ class WebSocket(object):
             raise err
 
         if proxy_host:
-            self._tunnel(hostname, port)
+            self._tunnel(hostname, port, proxy_auth)
 
         if is_secure:
             if HAVE_SSL:
@@ -461,9 +523,16 @@ class WebSocket(object):
 
         self._handshake(hostname, port, resource, **options)
 
-    def _tunnel(self, host, port):
+    def _tunnel(self, host, port, auth):
         logger.debug("Connecting proxy...")
         connect_header = "CONNECT %s:%d HTTP/1.0\r\n" % (host, port)
+        # TODO: support digest auth.
+        if auth and auth[0]:
+            auth_str = auth[0]
+            if auth[1]:
+                auth_str += ":" + auth[1]
+            encoded_str = base64encode(auth_str.encode()).strip().decode()
+            connect_header += "Proxy-Authorization: Basic %s\r\n" % encoded_str
         connect_header += "\r\n"
         _dump("request header", connect_header)
 
@@ -818,11 +887,19 @@ class WebSocket(object):
 
         self.shutdown()
 
+    def abort(self):
+        """
+        Low-level asynchonous abort, wakes up other threads that are waiting in recv_*
+        """
+        if self.connected:
+            self.sock.shutdown(socket.SHUT_RDWR)
+
     def shutdown(self):
         "close socket, immediately."
         if self.sock:
             self.sock.close()
             self.sock = None
+            self.connected = False
 
     def _send(self, data):
         if isinstance(data, six.text_type):
@@ -834,11 +911,11 @@ class WebSocket(object):
         try:
             return self.sock.send(data)
         except socket.timeout as e:
-            message = getattr(e, 'strerror', getattr(e, 'message', ''))
+            message = _extract_err_message(e)
             raise WebSocketTimeoutException(message)
         except Exception as e:
-            message = getattr(e, 'strerror', getattr(e, 'message', ''))
-            if "timed out" in message:
+            message = _extract_err_message(e)
+            if message and "timed out" in message:
                 raise WebSocketTimeoutException(message)
             else:
                 raise
@@ -850,16 +927,19 @@ class WebSocket(object):
         try:
             bytes = self.sock.recv(bufsize)
         except socket.timeout as e:
-            message = getattr(e, 'strerror', getattr(e, 'message', ''))
+            message = _extract_err_message(e)
             raise WebSocketTimeoutException(message)
         except SSLError as e:
-            message = getattr(e, 'strerror', getattr(e, 'message', ''))
+            message = _extract_err_message(e)
             if message == "The read operation timed out":
                 raise WebSocketTimeoutException(message)
             else:
                 raise
 
         if not bytes:
+            self.sock.close()
+            self.sock = None
+            self.connected = False
             raise WebSocketConnectionClosedException()
         return bytes
 
